@@ -15,22 +15,8 @@
  * Architecture
  */
 #ifndef ARCH
-#ifdef __SYMBIAN32__
-#if defined(__WINS__)
-#define ARCH "WINS"
-#elif defined(__MARM__)
-#define ARCH "ARM"
-#elif defined(__MARM_ARMI__)
-#define ARCH "ARMI"
-#elif defined(EKA2)
-#define ARCH "EKA2"
-#else
-#define ARCH "Symbian Arch"
-#endif
-#else
 #define ARCH "?"
 #endif
-#endif
 
 /**
  * @def OS
@@ -38,9 +24,7 @@
  * Operating System
  */
 #ifndef OS
-#ifdef __SYMBIAN32__
-#define OS "Symbian"
-#elif defined (WIN32)
+#ifdef WIN32
 #define OS "win32"
 #else
 #define OS "?"
@@ -87,3 +71,4 @@ void     rand_bytes(uint8_t *p, size_t size);
 /* File-System */
 int  fs_mkdir(const char *path, uint16_t mode);
 int  fs_gethome(char *path, size_t sz);
+bool fs_isdir(const char *path);
