@@ -30,7 +30,7 @@ static void destructor(void *arg)
 
 int aes_alloc(struct aes **stp, enum aes_mode mode,
 	      const uint8_t *key, size_t key_bits,
-	      const uint8_t iv[AES_BLOCK_SIZE])
+	      const uint8_t *iv)
 {
 	struct aes *st;
 	size_t key_bytes = key_bits / 8;
@@ -75,7 +75,7 @@ int aes_alloc(struct aes **stp, enum aes_mode mode,
 }
 
 
-void aes_set_iv(struct aes *st, const uint8_t iv[AES_BLOCK_SIZE])
+void aes_set_iv(struct aes *st, const uint8_t *iv)
 {
 	CCCryptorStatus status;
 
@@ -123,3 +123,23 @@ int aes_decr(struct aes *st, uint8_t *out, const uint8_t *in, size_t len)
 {
 	return aes_encr(st, out, in, len);
 }
+
+
+int aes_get_authtag(struct aes *aes, uint8_t *tag, size_t taglen)
+{
+	(void)aes;
+	(void)tag;
+	(void)taglen;
+
+	return ENOSYS;
+}
+
+
+int aes_authenticate(struct aes *aes, const uint8_t *tag, size_t taglen)
+{
+	(void)aes;
+	(void)tag;
+	(void)taglen;
+
+	return ENOSYS;
+}
