@@ -876,10 +876,12 @@ int dnsc_alloc(struct dnsc **dcpp, const struct dnsc_conf *conf,
 		goto out;
 
 	sa_set_str(&laddr, "0.0.0.0", 0);
-	sa_set_str(&laddr6, "::", 0);
-
 	err  = udp_listen(&dnsc->us, &laddr, udp_recv_handler, dnsc);
+
+#ifdef HAVE_INET6
+	sa_set_str(&laddr6, "::", 0);
 	err |= udp_listen(&dnsc->us6, &laddr6, udp_recv_handler, dnsc);
+#endif
 	if (err)
 		goto out;
 
@@ -901,6 +903,33 @@ int dnsc_alloc(struct dnsc **dcpp, const struct dnsc_conf *conf,
 }
 
 
+int dnsc_conf_set(struct dnsc *dnsc, const struct dnsc_conf *conf)
+{
+	int err;
+	if (!dnsc)
+		return EINVAL;
+
+	if (conf)
+		dnsc->conf = *conf;
+	else
+		dnsc->conf = default_conf;
+
+
+	dnsc->ht_query = mem_deref(dnsc->ht_query);
+	dnsc->ht_tcpconn = mem_deref(dnsc->ht_tcpconn);
+
+	err = hash_alloc(&dnsc->ht_query, dnsc->conf.query_hash_size);
+	if (err)
+		return err;
+
+	err = hash_alloc(&dnsc->ht_tcpconn, dnsc->conf.tcp_hash_size);
+	if (err)
+		return err;
+
+	return err;
+}
+
+
 /**
  * Set the DNS Servers on a DNS Client
  *
