@@ -22,20 +22,30 @@
 #include "http.h"
 
 
+#define DEBUG_MODULE "http_client"
+#define DEBUG_LEVEL 5
+#include <re_dbg.h>
+
+
 enum {
 	CONN_TIMEOUT = 30000,
 	RECV_TIMEOUT = 60000,
 	IDLE_TIMEOUT = 900000,
 	BUFSIZE_MAX  = 524288,
 	CONN_BSIZE   = 256,
+	QUERY_HASH_SIZE = 16,
+	TCP_HASH_SIZE = 2,
 };
 
 struct http_cli {
+	struct http_conf conf;
 	struct list reql;
 	struct hash *ht_conn;
 	struct dnsc *dnsc;
 	struct tls *tls;
-	char *tls_hostname;
+	char *tlshn;
+	char *cert;
+	char *key;
 	struct sa laddr;
 #ifdef HAVE_INET6
 	struct sa laddr6;
@@ -69,6 +79,13 @@ struct http_req {
 };
 
 
+static const struct http_conf default_conf = {
+	CONN_TIMEOUT,
+	RECV_TIMEOUT,
+	IDLE_TIMEOUT,
+};
+
+
 struct conn {
 	struct tmr tmr;
 	struct sa addr;
@@ -100,9 +117,11 @@ static void cli_destructor(void *arg)
 
 	hash_flush(cli->ht_conn);
 	mem_deref(cli->ht_conn);
+	mem_deref(cli->cert);
+	mem_deref(cli->key);
 	mem_deref(cli->dnsc);
 	mem_deref(cli->tls);
-	mem_deref(cli->tls_hostname);
+	mem_deref(cli->tlshn);
 }
 
 
@@ -133,7 +152,23 @@ static void conn_destructor(void *arg)
 
 static void conn_idle(struct conn *conn)
 {
-	tmr_start(&conn->tmr, IDLE_TIMEOUT, timeout_handler, conn);
+	struct http_req *req;
+	struct http_cli *cli;
+
+	if (!conn)
+		return;
+
+	req = conn->req;
+	if (!req)
+		goto out;
+
+	cli = req->cli;
+	if (!cli)
+		goto out;
+
+	tmr_start(&conn->tmr, cli->conf.idle_timeout, timeout_handler, conn);
+
+ out:
 	conn->req = NULL;
 }
 
@@ -288,6 +323,7 @@ static void estab_handler(void *arg)
 {
 	struct conn *conn = arg;
 	struct http_req *req = conn->req;
+	struct http_cli *cli;
 	int err;
 
 	if (!req)
@@ -299,7 +335,11 @@ static void estab_handler(void *arg)
 		return;
 	}
 
-	tmr_start(&conn->tmr, RECV_TIMEOUT, timeout_handler, conn);
+	cli = req->cli;
+	if (!cli)
+		return;
+
+	tmr_start(&conn->tmr, cli->conf.recv_timeout, timeout_handler, conn);
 }
 
 
@@ -408,6 +448,7 @@ static int conn_connect(struct http_req *req)
 	const struct sa *addr = &req->srvv[req->srvc];
 	struct conn *conn;
 	struct sa *laddr = NULL;
+	struct http_cli *cli;
 	int err;
 
 	conn = list_ledata(hash_lookup(req->cli->ht_conn,
@@ -415,7 +456,11 @@ static int conn_connect(struct http_req *req)
 	if (conn) {
 		err = tcp_send(conn->tc, req->mbreq);
 		if (!err) {
-			tmr_start(&conn->tmr, RECV_TIMEOUT,
+			cli = req->cli;
+			if (!cli)
+				return EINVAL;
+
+			tmr_start(&conn->tmr, cli->conf.recv_timeout,
 				  timeout_handler, conn);
 
 			req->conn = conn;
@@ -461,20 +506,19 @@ static int conn_connect(struct http_req *req)
 		if (err)
 			goto out;
 
-		if (req->cli->tls_hostname)
-			err = tls_peer_set_verify_host(conn->sc,
-				req->cli->tls_hostname);
+		if (req->cli->tlshn)
+			err  = tls_set_verify_server(conn->sc,
+					req->cli->tlshn);
+		else
+			err  = tls_set_verify_server(conn->sc, req->host);
 
 		if (err)
 			goto out;
-
-		err = tls_set_servername(conn->sc, req->host);
-		if (err)
-			goto out;
 	}
 #endif
 
-	tmr_start(&conn->tmr, CONN_TIMEOUT, timeout_handler, conn);
+	tmr_start(&conn->tmr, req->cli->conf.conn_timeout, timeout_handler,
+		  conn);
 
 	req->conn = conn;
 	conn->req = req;
@@ -556,6 +600,76 @@ static void query_handler(int err, const struct dnshdr *hdr, struct list *ansl,
 }
 
 
+#ifdef USE_TLS
+static int read_file(char **pbuf, const char *path)
+{
+	FILE *f = NULL;
+	size_t s = 0;
+	size_t n = 0;
+	char *buf;
+
+	if (!pbuf || !path)
+		return EINVAL;
+
+	f = fopen(path, "r");
+	if (!f) {
+		DEBUG_WARNING("Could not open cert file '%s'\n", path);
+		return EIO;
+	}
+
+	fseek(f, 0L, SEEK_END);
+	s = ftell(f);
+	fseek(f, 0L, SEEK_SET);
+
+	buf = mem_alloc(s + 1, NULL);
+	if (!buf) {
+		DEBUG_WARNING("Could not allocate cert file buffer\n");
+		fclose(f);
+		return ENOMEM;
+	}
+
+	n = fread(buf, 1, s, f);
+	fclose(f);
+	buf[s] = 0;
+	if (n < s) {
+		buf = mem_deref(buf);
+		return EIO;
+	}
+
+	*pbuf = buf;
+	return 0;
+}
+#endif
+
+
+int http_uri_decode(struct http_uri *hu, const struct pl *uri)
+{
+	int err = 0;
+	if (!hu)
+		return EINVAL;
+
+	memset(hu, 0, sizeof(*hu));
+
+	/* Try IPv6 first */
+	err = re_regex(uri->p, uri->l, "[a-z]+://\\[[^\\]]+\\][:]*[0-9]*[^]+",
+		       &hu->scheme, &hu->host, NULL, &hu->port, &hu->path) ||
+	      hu->scheme.p != uri->p;
+	if (!err)
+		goto out;
+
+	/* Then non-IPv6 host */
+	err = re_regex(uri->p, uri->l, "[a-z]+://[^:/]+[:]*[0-9]*[^]+",
+		       &hu->scheme, &hu->host, NULL, &hu->port, &hu->path) ||
+	      hu->scheme.p != uri->p;
+
+out:
+	if (!err && !pl_isset(&hu->path))
+		pl_set_str(&hu->path, "/");
+
+	return err;
+}
+
+
 /**
  * Send an HTTP request
  *
@@ -574,7 +688,8 @@ int http_request(struct http_req **reqp, struct http_cli *cli, const char *met,
 		 const char *uri, http_resp_h *resph, http_data_h *datah,
 		 void *arg, const char *fmt, ...)
 {
-	struct pl scheme, host, port, path;
+	struct http_uri http_uri;
+	struct pl pl;
 	struct http_req *req;
 	uint16_t defport;
 	bool secure;
@@ -584,18 +699,18 @@ int http_request(struct http_req **reqp, struct http_cli *cli, const char *met,
 	if (!cli || !met || !uri)
 		return EINVAL;
 
-	if (re_regex(uri, strlen(uri), "[a-z]+://[^:/]+[:]*[0-9]*[^]+",
-		     &scheme, &host, NULL, &port, &path) || scheme.p != uri)
+	pl_set_str(&pl, uri);
+	if (http_uri_decode(&http_uri, &pl))
 		return EINVAL;
 
-	if (!pl_strcasecmp(&scheme, "http") ||
-	    !pl_strcasecmp(&scheme, "ws")) {
+	if (!pl_strcasecmp(&http_uri.scheme, "http") ||
+	    !pl_strcasecmp(&http_uri.scheme, "ws")) {
 		secure  = false;
 		defport = 80;
 	}
 #ifdef USE_TLS
-	else if (!pl_strcasecmp(&scheme, "https") ||
-		 !pl_strcasecmp(&scheme, "wss")) {
+	else if (!pl_strcasecmp(&http_uri.scheme, "https") ||
+		 !pl_strcasecmp(&http_uri.scheme, "wss")) {
 		secure  = true;
 		defport = 443;
 	}
@@ -611,12 +726,13 @@ int http_request(struct http_req **reqp, struct http_cli *cli, const char *met,
 
 	req->cli    = cli;
 	req->secure = secure;
-	req->port   = pl_isset(&port) ? pl_u32(&port) : defport;
+	req->port   = pl_isset(&http_uri.port) ? pl_u32(&http_uri.port) :
+			defport;
 	req->resph  = resph;
 	req->datah  = datah;
 	req->arg    = arg;
 
-	err = pl_strdup(&req->host, &host);
+	err = pl_strdup(&req->host, &http_uri.host);
 	if (err)
 		goto out;
 
@@ -629,7 +745,7 @@ int http_request(struct http_req **reqp, struct http_cli *cli, const char *met,
 	err = mbuf_printf(req->mbreq,
 			  "%s %r HTTP/1.1\r\n"
 			  "Host: %r\r\n",
-			  met, &path, &host);
+			  met, &http_uri.path, &http_uri.host);
 	if (fmt) {
 		va_start(ap, fmt);
 		err |= mbuf_vprintf(req->mbreq, fmt, ap);
@@ -643,6 +759,18 @@ int http_request(struct http_req **reqp, struct http_cli *cli, const char *met,
 
 	req->mbreq->pos = 0;
 
+#ifdef USE_TLS
+	if (cli->cert && cli->key) {
+		err = tls_set_certificate_pem(cli->tls,
+				cli->cert, strlen(cli->cert),
+				cli->key, strlen(cli->key));
+	}
+	else if (cli->cert) {
+		err = tls_set_certificate(cli->tls,
+				cli->cert, strlen(cli->cert));
+	}
+#endif
+
 	if (!sa_set_str(&req->srvv[0], req->host, req->port)) {
 
 		req->srvc = 1;
@@ -686,10 +814,27 @@ void http_req_set_conn_handler(struct http_req *req, http_conn_h *connh)
 }
 
 
+int http_client_set_config(struct http_cli *cli, struct http_conf *conf)
+{
+	struct dnsc_conf dconf;
+	if (!cli || !conf)
+		return EINVAL;
+
+	cli->conf = *conf;
+
+	dconf.query_hash_size = QUERY_HASH_SIZE;
+	dconf.tcp_hash_size = TCP_HASH_SIZE;
+	dconf.conn_timeout = conf->conn_timeout;
+	dconf.idle_timeout = conf->idle_timeout;
+
+	return dnsc_conf_set(cli->dnsc, &dconf);
+}
+
+
 /**
- * Allocate an HTTP client instance
+ * Allocate an HTTP Client instance
  *
- * @param clip      Pointer to allocated HTTP client
+ * @param clip      Pointer to allocated HTTP Client
  * @param dnsc      DNS Client
  *
  * @return 0 if success, otherwise errorcode
@@ -723,6 +868,7 @@ int http_client_alloc(struct http_cli **clip, struct dnsc *dnsc)
 #endif
 
 	cli->dnsc = mem_ref(dnsc);
+	cli->conf = default_conf;
 
  out:
 	if (err)
@@ -738,7 +884,7 @@ int http_client_alloc(struct http_cli **clip, struct dnsc *dnsc)
 /**
  * Add trusted CA certificates
  *
- * @param cli     HTTP client
+ * @param cli     HTTP Client
  * @param capath  Path to CA certificates
  *
  * @return 0 if success, otherwise errorcode
@@ -752,10 +898,108 @@ int http_client_add_ca(struct http_cli *cli, const char *tls_ca)
 }
 
 
+/**
+ * Add trusted CA certificates given as string
+ *
+ * @param cli    HTTP Client
+ * @param capem  The trusted CA as 0-terminated string given in PEM format
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int http_client_add_capem(struct http_cli *cli, const char *capem)
+{
+	if (!cli || !capem)
+		return EINVAL;
+
+	return tls_add_capem(cli->tls, capem);
+}
+
+
+/**
+ * Set client certificate
+ * @param cli   HTTP Client
+ * @param path  File path to client certificate
+ *
+ * @return 0 for success, error code otherwise.
+ */
+int http_client_set_cert(struct http_cli *cli, const char *path)
+{
+	int err = 0;
+
+	if (!cli || !path)
+		return EINVAL;
+
+	cli->cert = mem_deref(cli->cert);
+	err = read_file(&cli->cert, path);
+	if (err) {
+		cli->cert = mem_deref(cli->cert);
+		return err;
+	}
+
+	return 0;
+}
+
+
+/**
+ * Set client certificate in PEM format
+ * @param cli    HTTP Client
+ * @param pem    Client certificate in PEM format
+ *
+ * @return 0 for success, error code otherwise.
+ */
+/* ------------------------------------------------------------------------- */
+int http_client_set_certpem(struct http_cli *cli, const char *pem)
+{
+	if (!cli || !str_isset(pem))
+		return EINVAL;
+
+	cli->cert = mem_deref(cli->cert);
+	cli->cert = mem_zalloc(strlen(pem) + 1, NULL);
+	if (!cli->cert)
+		return ENOMEM;
+
+	strcpy(cli->cert, pem);
+	return 0;
+}
+
+
+int http_client_set_key(struct http_cli *cli, const char *path)
+{
+	int err = 0;
+
+	if (!cli || !path)
+		return EINVAL;
+
+	cli->key = mem_deref(cli->key);
+	err = read_file(&cli->key, path);
+	if (err) {
+		cli->key = mem_deref(cli->key);
+		return err;
+	}
+
+	return 0;
+}
+
+
+int http_client_set_keypem(struct http_cli *cli, const char *pem)
+{
+	if (!cli || !str_isset(pem))
+		return EINVAL;
+
+	cli->key = mem_deref(cli->key);
+	cli->key = mem_zalloc(strlen(pem) + 1, NULL);
+	if (!cli->key)
+		return ENOMEM;
+
+	strcpy(cli->key, pem);
+	return 0;
+}
+
+
 /**
  * Set verify host name
  *
- * @param cli       HTTP client
+ * @param cli       HTTP Client
  * @param hostname  String for alternative name validation.
  *
  * @return 0 if success, otherwise errorcode
@@ -763,10 +1007,14 @@ int http_client_add_ca(struct http_cli *cli, const char *tls_ca)
 int http_client_set_tls_hostname(struct http_cli *cli,
 				 const struct pl *hostname)
 {
-	if (!cli || !hostname)
+	if (!cli)
 		return EINVAL;
 
-	return tls_set_hostname(cli->tls_hostname, hostname);
+	cli->tlshn = mem_deref(cli->tlshn);
+	if (!hostname)
+		return 0;
+
+	return pl_strdup(&cli->tlshn, hostname);
 }
 #endif
 
@@ -778,7 +1026,7 @@ int http_client_set_tls_hostname(struct http_cli *cli,
  * @param addr  Bind to local v4 address
  *
  */
-void http_client_set_laddr(struct http_cli *cli, struct sa *addr)
+void http_client_set_laddr(struct http_cli *cli, const struct sa *addr)
 {
 	if (cli && addr)
 		sa_cpy(&cli->laddr, addr);
@@ -792,7 +1040,7 @@ void http_client_set_laddr(struct http_cli *cli, struct sa *addr)
  * @param addr   Bind to local v6 address
  *
  */
-void http_client_set_laddr6(struct http_cli *cli, struct sa *addr)
+void http_client_set_laddr6(struct http_cli *cli, const struct sa *addr)
 {
 #ifdef HAVE_INET6
 	if (cli && addr)
