@@ -114,8 +114,9 @@ static void sigpipe_handler(int x)
 
 int openssl_init(void)
 {
+	int err;
 #if defined (HAVE_PTHREAD) && (OPENSSL_VERSION_NUMBER < 0x10100000L)
-	int err, i;
+	int i;
 
 	lockv = mem_zalloc(sizeof(pthread_mutex_t) * CRYPTO_num_locks(), NULL);
 	if (!lockv)
@@ -150,7 +151,6 @@ int openssl_init(void)
 #endif
 
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
-	int err;
 	err = OPENSSL_init_ssl(OPENSSL_INIT_SSL_DEFAULT, NULL);
 	if (!err)
 		return !err;
