@@ -345,6 +345,39 @@ int rtp_listen(struct rtp_sock **rsp, int proto, const struct sa *ip,
 }
 
 
+/**
+ * Open RTP Socket without bind.
+ *
+ * @param rsp         Pointer to returned RTP socket
+ * @param af          Address family AF_INET or AF_INET6
+ *
+ * @return 0 for success, otherwise errorcode
+ */
+int rtp_open(struct rtp_sock **rsp, int af)
+{
+	struct rtp_sock *rs;
+	struct udp_sock *us_rtp;
+	int err;
+
+	err = rtp_alloc(&rs);
+	if (err)
+		return err;
+
+	rs->proto = IPPROTO_UDP;
+
+	us_rtp = NULL;
+	err = udp_open(&us_rtp, af);
+	rs->sock_rtp = us_rtp;
+
+	if (err)
+		mem_deref(rs);
+	else
+		*rsp = rs;
+
+	return err;
+}
+
+
 /**
  * Encode a new RTP header into the beginning of the buffer
  *
