@@ -182,6 +182,7 @@ static struct sip_conn *ws_conn_find(struct sip *sip, const struct sa *paddr,
 				     enum sip_transp tp)
 {
 	struct le *le;
+	(void) tp;
 
 	le = list_head(hash_list(sip->ht_conn, sa_hash(paddr, SA_ALL)));
 
@@ -631,13 +632,17 @@ static void tcp_connect_handler(const struct sa *paddr, void *arg)
 
 
 static int conn_send(struct sip_connqent **qentp, struct sip *sip, bool secure,
-		     const struct sa *dst, struct mbuf *mb,
+		     const struct sa *dst, char *host, struct mbuf *mb,
 		     sip_transp_h *transph, void *arg)
 {
 	struct sip_conn *conn, *new_conn = NULL;
 	struct sip_connqent *qent;
 	int err = 0;
 
+#ifndef USE_TLS
+	(void) host;
+#endif
+
 	conn = conn_find(sip, dst, secure);
 	if (conn) {
 		if (!conn->established)
@@ -682,6 +687,10 @@ static int conn_send(struct sip_connqent **qentp, struct sip *sip, bool secure,
 		err = tls_start_tcp(&conn->sc, transp->tls, conn->tc, 0);
 		if (err)
 			goto out;
+
+		err = tls_set_verify_server(conn->sc, host);
+		if (err)
+			goto out;
 	}
 #endif
 
@@ -768,6 +777,7 @@ static void websock_recv_handler(const struct websock_hdr *hdr,
 	struct sip_msg *msg;
 	size_t start;
 	int err;
+	(void) hdr;
 
 #if 0
 	re_printf(
@@ -1153,8 +1163,8 @@ void sip_transp_flush(struct sip *sip)
 
 
 int sip_transp_send(struct sip_connqent **qentp, struct sip *sip, void *sock,
-		    enum sip_transp tp, const struct sa *dst, struct mbuf *mb,
-		    sip_transp_h *transph, void *arg)
+		    enum sip_transp tp, const struct sa *dst, char *host,
+		    struct mbuf *mb, sip_transp_h *transph, void *arg)
 {
 	const struct sip_transport *transp;
 	struct sip_conn *conn;
@@ -1194,7 +1204,7 @@ int sip_transp_send(struct sip_connqent **qentp, struct sip *sip, void *sock,
 			err = tcp_send(conn->tc, mb);
 		}
 		else
-			err = conn_send(qentp, sip, secure, dst, mb,
+			err = conn_send(qentp, sip, secure, dst, host, mb,
 					transph, arg);
 		break;
 
@@ -1350,6 +1360,24 @@ const char *sip_transp_param(enum sip_transp tp)
 }
 
 
+enum sip_transp sip_transp_decode(const struct pl *pl)
+{
+	enum sip_transp tp = SIP_TRANSP_NONE;
+	if (!pl_strcasecmp(pl, "udp"))
+		tp = SIP_TRANSP_UDP;
+	else if (!pl_strcasecmp(pl, "tcp"))
+		tp = SIP_TRANSP_TCP;
+	else if (!pl_strcasecmp(pl, "tls"))
+		tp = SIP_TRANSP_TLS;
+	else if (!pl_strcasecmp(pl, "ws"))
+		tp = SIP_TRANSP_WS;
+	else if (!pl_strcasecmp(pl, "wss"))
+		tp = SIP_TRANSP_WSS;
+
+	return tp;
+}
+
+
 bool sip_transp_reliable(enum sip_transp tp)
 {
 	switch (tp) {
