@@ -65,7 +65,8 @@ int uri_encode(struct re_printf *pf, const struct uri *uri)
 	if (uri->port)
 		err = re_hprintf(pf, ":%u", uri->port);
 
-	err |= re_hprintf(pf, "%r%r", &uri->params, &uri->headers);
+	err |= re_hprintf(pf, "%r%r%r", &uri->path, &uri->params,
+			  &uri->headers);
 
 	return err;
 }
@@ -117,17 +118,19 @@ int uri_decode(struct uri *uri, const struct pl *pl)
 
 	memset(uri, 0, sizeof(*uri));
 	if (0 == re_regex(pl->p, pl->l,
-			  "[^:]+:[^@:]*[:]*[^@]*@[^;? ]+[^?]*[^]*",
+			  "[^:]+:[^@:]*[:]*[^@]*@[^/;? ]+[^;? ]*[^?]*[^]*",
 			  &uri->scheme, &uri->user, NULL, &uri->password,
-			  &hostport, &uri->params, &uri->headers)) {
+			  &hostport, &uri->path, &uri->params,
+			  &uri->headers)) {
 
 		if (0 == uri_decode_hostport(&hostport, &uri->host, &port))
 			goto out;
 	}
 
 	memset(uri, 0, sizeof(*uri));
-	err = re_regex(pl->p, pl->l, "[^:]+:[^;? ]+[^?]*[^]*",
-		       &uri->scheme, &hostport, &uri->params, &uri->headers);
+	err = re_regex(pl->p, pl->l, "[^:]+:[^/;? ]+[^;? ]*[^?]*[^]*",
+		       &uri->scheme, &hostport, &uri->path, &uri->params,
+		       &uri->headers);
 	if (0 == err) {
 		err = uri_decode_hostport(&hostport, &uri->host, &port);
 		if (0 == err)
